#!/bin/sh
#
# NFS service

. /etc/service.subr

prog_dir=`dirname \`realpath $0\``

name="nfs"
version="1.2.8"
description="Kernel-based NFS server"
framework_version="2.0"

rpcbind="${prog_dir}/bin/rpcbind"
mountd="${prog_dir}/sbin/rpc.mountd"
statd="${prog_dir}/sbin/rpc.statd"
smnotify="${prog_dir}/sbin/sm-notify"

# pidfile = rpc.mountd.pid
pidfile="/tmp/DroboApps/${name}/pid.txt"
bnpidfile="/tmp/DroboApps/${name}/rpcbind.pid"
stpidfile="/tmp/DroboApps/${name}/rpc.statd.pid"
smpidfile="/tmp/DroboApps/${name}/sm-notify.pid"
lockfile="/tmp/DroboApps/${name}/rpcbind.lock"
statuser=nobody
logfile="/tmp/DroboApps/${name}/log.txt"
statusfile="/tmp/DroboApps/${name}/status.txt"
errorfile="/tmp/DroboApps/${name}/error.txt"
exportsfile="${prog_dir}/etc/exports"

_fix_dev_permissions() {
  chmod a+rw /dev/null /dev/full /dev/random /dev/urandom /dev/tty /dev/ptmx /dev/zero 1>> "${logfile}" 2>&1
}

_create_exports() {
  local ipaddr network netmask
  if [ ! -f "${exportsfile}" ]; then
    cp "${exportsfile}.default" "${exportsfile}" 1>> "${logfile}" 2>&1
    ipaddr=`ip address show dev eth0 | grep "inet " | awk '{print $2}'`
    network=`echo $ipaddr | awk -F. '{print $1 "." $2 "." $3 "." 0}'`
    netmask=`echo $ipaddr | awk -F/ '{print $2}'`
    sed -i -e "s|NETMASK_GOES_HERE|${network}/${netmask}|g" "${exportsfile}" 1>> "${logfile}" 2>&1
  fi
}

_create_config() {
  if [ ! -f "/etc/netconfig" ]; then
    cp "${prog_dir}/etc/netconfig.default" "/etc/netconfig" 1>> "${logfile}" 2>&1
  fi
}

_fix_permissions() {
  chmod 4511 "${prog_dir}/sbin/mount.nfs"
  chown -R "${statuser}" "${prog_dir}/var/lib/nfs/sm" "${prog_dir}/var/lib/nfs/sm.bak" "${prog_dir}/var/lib/nfs/state" "${prog_dir}/var/lib/nfs/statd"
}

# $1 modulename
# $2 filename
_module_load() {
  /sbin/lsmod | grep -q "^$1" 2> /dev/null
  if [ $? -ne 0 ]; then
    /sbin/insmod "$2" 1>> "${logfile}" 2>&1
  fi
}

# $1 modulename
# $2 filename
_module_unload() {
  /sbin/lsmod | grep -q "^$1" 1> /dev/null 2>&1
  if [ $? -eq 0 ]; then
    /sbin/rmmod -r "$2" 1>> "${logfile}" 2>&1
  fi
}

_mount_nfsd() {
  grep -q ^nfsd /proc/mounts 2> /dev/null
  if [ $? -ne 0 ]; then
    mount -t nfsd nfsd /proc/fs/nfsd 1> /dev/null 2>&1
  fi
}

_avahi_publish() {
  local exports name file
  rm -f "${prog_dir}/etc/nfs-*.service" 1>> "${logfile}" 2>&1
  if [ -f "${exportsfile}" ]; then
    exports=`cat "${exportsfile}" | awk '{print $1}'`
    for p in $exports; do
      name=`basename "${p}"`
      file="${prog_dir}/etc/nfs-${name}.service"
      sed -e "s|NAME_GOES_HERE|${name}|" -e "s|PATH_GOES_HERE|${p}|" "${prog_dir}/etc/nfs.service.template" > "${file}"
      ln -fs "${file}" "/etc/avahi/services/" 1>> "${logfile}" 2>&1
    done
    avahi-daemon -r 1>> "${logfile}" 2>&1
  fi
}

_avahi_clear() {
  rm -f /etc/avahi/services/nfs-*.service 1>> "${logfile}" 2>&1
  avahi-daemon -r 1>> "${logfile}" 2>&1
}

# $1 pidfile
# $2 filename
_is_process_running() {
  local pid
  pid=`cat "$1" 2> /dev/null`
    if [ -z "${pid}" ]; then return 1; fi
  kill -0 "${pid}" 2> /dev/null
    if [ $? -ne 0 ]; then return 1; fi
  grep -q "$2" /proc/$pid/cmdline 2> /dev/null
    if [ $? -ne 0 ]; then return 1; fi
  return 0
}

# Overwrites the original definition
_is_running() {
  local pid
  _is_process_running "${bnpidfile}" "${rpcbind}"
    if [ $? -ne 0 ]; then return 1; fi
  _is_process_running "${pidfile}" "${mountd}"
    if [ $? -ne 0 ]; then return 1; fi
  _is_process_running "${stpidfile}" "${statd}"
    if [ $? -ne 0 ]; then return 1; fi
  killall -0 "nfsd"
    if [ $? -ne 0 ]; then return 1; fi
  return 0
}

# $1 filename
# $2 pidfile
_stop_process() {
  killall `basename "$1"` 1>> "${logfile}" 2>&1
  rm -f "$2"
}

# $1 filename
# $2 args
# $3 pidfile
_start_process() {
  _stop_process "$1" "$3"
  setsid "$1" $2 1>> "${logfile}" 2>&1 &
  if [ $! -gt 0 ]; then echo $! > "$3"; fi
}

_stop_nfsd() {
  killall -9 nfsd 1>> "${logfile}" 2>&1
}

_start_nfsd() {
  _stop_nfsd
  "${prog_dir}/sbin/rpc.nfsd" -p 2049 1>> "${logfile}" 2>&1
}

start() {
  _fix_dev_permissions
  _create_exports
  _create_config
  _fix_permissions
  _module_load "nfsd" "${prog_dir}/modules/nfsd.ko"
  _mount_nfsd
  reload_exports
  _stop_process "${smnotify}" "${smpidfile}"
  _start_process "${rpcbind}" "-d" "${bnpidfile}"
  _start_process "${mountd}" "-F" "${pidfile}"
  _start_process "${statd}" "-F" "${stpidfile}"
  _start_nfsd
  _avahi_publish
}

reload_exports() {
  "${prog_dir}/sbin/exportfs" -r -v 1>> "${logfile}" 2>&1
}

# Overwrites the original definition
stop_service() {
  _avahi_clear
  _stop_nfsd
  _stop_process "${statd}" "${stpidfile}"
  _stop_process "${mountd}" "${pidfile}"
  _stop_process "${smnotify}" "${smpidfile}"
  _stop_process "${rpcbind}" "${bnpidfile}"
  rm -f "${lockfile}"
  return 0
}

case "$1" in
  start)   start_service ;;
  stop)    stop_service ;;
  restart) stop_service ; sleep 3 ; start_service ;;
  reload)  reload_exports ;;
  status)  status ;;
  *) echo "Usage: $0 [start|stop|restart|reload|status]" ; exit 1 ;;
esac
